@@ -25,8 +25,8 @@ package Bugzilla::Install::Requirements;
 
 use strict;
 
+use Bugzilla::Install::Util qw(vers_cmp install_string);
 use List::Util qw(max);
-use POSIX ();
 use Safe;
 
 use base qw(Exporter);
@@ -36,9 +36,7 @@ our @EXPORT = qw(
 
     check_requirements
     check_graphviz
-    display_version_and_os
     have_vers
-    vers_cmp
     install_command
 );
 
@@ -63,27 +61,28 @@ sub REQUIRED_MODULES {
         module  => 'CGI',
         # Perl 5.10 requires CGI 3.33 due to a taint issue when
         # uploading attachments, see bug 416382.
-        version => (vers_cmp($perl_ver, '5.10') > -1) ? '3.33' : '2.93'
+        # Require CGI 3.21 for -httponly support, see bug 368502.
+        version => (vers_cmp($perl_ver, '5.10') > -1) ? '3.33' : '3.21'
     },
     {
         package => 'TimeDate',
         module  => 'Date::Format',
         version => '2.21'
     },
-    {
-        package => 'DBI',
-        module  => 'DBI',
-        version => '1.41'
-    },
     {
         package => 'PathTools',
         module  => 'File::Spec',
         version => '0.84'
     },
+    {
+        package => 'DBI',
+        module  => 'DBI',
+        version => '1.41'
+    },
     {
         package => 'Template-Toolkit',
         module  => 'Template',
-        version => '2.12'
+        version => '2.15'
     },
     {
         package => 'Email-Send',
@@ -91,10 +90,14 @@ sub REQUIRED_MODULES {
         version => ON_WINDOWS ? '2.16' : '2.00'
     },
     {
-        # This will pull in Email::MIME for us, also. 
+        package => 'Email-MIME',
+        module  => 'Email::MIME',
+        version => '1.861'
+    },
+    {
         package => 'Email-MIME-Modifier',
         module  => 'Email::MIME::Modifier',
-        version => 0
+        version => '1.442'
     },
     );
 
@@ -111,6 +114,12 @@ sub OPTIONAL_MODULES {
         version => '1.20',
         feature => 'Graphical Reports, New Charts, Old Charts'
     },
+    {
+        package => 'Chart',
+        module  => 'Chart::Base',
+        version => '1.0',
+        feature => 'New Charts, Old Charts'
+    },
     {
         package => 'Template-GD',
         # This module tells us whether or not Template-GD is installed
@@ -120,10 +129,10 @@ sub OPTIONAL_MODULES {
         feature => 'Graphical Reports'
     },
     {
-        package => 'Chart',
-        module  => 'Chart::Base',
-        version => '1.0',
-        feature => 'New Charts, Old Charts'
+        package => 'GDTextUtil',
+        module  => 'GD::Text',
+        version => 0,
+        feature => 'Graphical Reports'
     },
     {
         package => 'GDGraph',
@@ -131,12 +140,6 @@ sub OPTIONAL_MODULES {
         version => 0,
         feature => 'Graphical Reports'
     },
-    { 
-        package => 'GDTextUtil',
-        module  => 'GD::Text',
-        version => 0,
-        feature => 'Graphical Reports'
-    },
     {
         package => 'XML-Twig',
         module  => 'XML::Twig',
@@ -174,6 +177,18 @@ sub OPTIONAL_MODULES {
         version => 0,
         feature => 'LDAP Authentication'
     },
+    {
+        package => 'Authen-SASL',
+        module  => 'Authen::SASL',
+        version => 0,
+        feature => 'SMTP Authentication'
+    },
+    {
+        package => 'RadiusPerl',
+        module  => 'Authen::Radius',
+        version => 0,
+        feature => 'RADIUS Authentication'
+    },
     {
         package => 'SOAP-Lite',
         module  => 'SOAP::Lite',
@@ -265,11 +280,11 @@ sub _get_extension_requirements {
 sub check_requirements {
     my ($output) = @_;
 
-    print "\nChecking perl modules...\n" if $output;
+    print "\n", install_string('checking_modules'), "\n" if $output;
     my $root = ROOT_USER;
-    my %missing = _check_missing(REQUIRED_MODULES, $output);
+    my $missing = _check_missing(REQUIRED_MODULES, $output);
 
-    print "\nChecking available perl DBD modules...\n" if $output;
+    print "\n", install_string('checking_dbd'), "\n" if $output;
     my $have_one_dbd = 0;
     my $db_modules = DB_MODULE;
     foreach my $db (keys %$db_modules) {
@@ -277,20 +292,20 @@ sub check_requirements {
         $have_one_dbd = 1 if have_vers($dbd, $output);
     }
 
-    print "\nThe following Perl modules are optional:\n" if $output;
-    my %missing_optional = _check_missing(OPTIONAL_MODULES, $output);
+    print "\n", install_string('checking_optional'), "\n" if $output;
+    my $missing_optional = _check_missing(OPTIONAL_MODULES, $output);
 
     # If we're running on Windows, reset the input line terminator so that
     # console input works properly - loading CGI tends to mess it up
     $/ = "\015\012" if ON_WINDOWS;
 
-    my $pass = !scalar(keys %missing) && $have_one_dbd;
+    my $pass = !scalar(@$missing) && $have_one_dbd;
     return {
         pass     => $pass,
         one_dbd  => $have_one_dbd,
-        missing  => \%missing,
-        optional => \%missing_optional,
-        any_missing => !$pass || scalar(keys %missing_optional),
+        missing  => $missing,
+        optional => $missing_optional,
+        any_missing => !$pass || scalar(@$missing_optional),
     };
 }
 
@@ -298,14 +313,14 @@ sub check_requirements {
 sub _check_missing {
     my ($modules, $output) = @_;
 
-    my %missing;
+    my @missing;
     foreach my $module (@$modules) {
         unless (have_vers($module, $output)) {
-            $missing{$module->{package}} = $module;
+            push(@missing, $module);
         }
     }
 
-    return %missing;
+    return \@missing;
 }
 
 # Returns the build ID of ActivePerl. If several versions of
@@ -323,13 +338,15 @@ sub print_module_instructions {
     my ($check_results, $output) = @_;
 
     # We only print these notes if we have to.
-    if ((!$output && %{$check_results->{missing}})
+    if ((!$output && @{$check_results->{missing}})
         || ($output && $check_results->{any_missing}))
     {
-        print "\n* NOTE: You must run any commands listed below as "
-              . ROOT_USER . ".\n\n";
-
+        
         if (ON_WINDOWS) {
+
+            print "\n* NOTE: You must run any commands listed below as "
+                  . ROOT_USER . ".\n\n";
+
             my $perl_ver = sprintf('%vd', $^V);
             
             # URL when running Perl 5.8.x.
@@ -367,7 +384,7 @@ EOT
     }
 
     # Required Modules
-    if (my %missing = %{$check_results->{missing}}) {
+    if (my @missing = @{$check_results->{missing}}) {
         print <<EOT;
 ***********************************************************************
 * REQUIRED MODULES                                                    *
@@ -381,8 +398,8 @@ EOT
 EOT
 
         print "COMMANDS:\n\n";
-        foreach my $package (keys %missing) {
-            my $command = install_command($missing{$package});
+        foreach my $package (@missing) {
+            my $command = install_command($package);
             print "    $command\n";
         }
         print "\n";
@@ -416,7 +433,7 @@ EOT
 
     return unless $output;
 
-    if (my %missing = %{$check_results->{optional}}) {
+    if (my @missing = @{$check_results->{optional}}) {
         print <<EOT;
 **********************************************************************
 * OPTIONAL MODULES                                                   *
@@ -432,12 +449,8 @@ EOT
 **********************************************************************
 
 EOT
-        # We want to sort them so that they are ordered by feature.
-        my @missing_names = sort {$missing{$a}->{feature} 
-                                  cmp $missing{$b}->{feature}} (keys %missing);
-
         # Now we have to determine how large the table cols will be.
-        my $longest_name = max(map(length($_), @missing_names));
+        my $longest_name = max(map(length($_->{package}), @missing));
 
         # The first column header is at least 11 characters long.
         $longest_name = 11 if $longest_name < 11;
@@ -450,18 +463,22 @@ EOT
         printf "* \%${longest_name}s * %-${remaining_space}s *\n",
                'MODULE NAME', 'ENABLES FEATURE(S)';
         print '*' x 71 . "\n";
-        foreach my $name (@missing_names) {
+        foreach my $package (@missing) {
             printf "* \%${longest_name}s * %-${remaining_space}s *\n",
-                   $name, $missing{$name}->{feature};
+                   $package->{package}, $package->{feature};
         }
         print '*' x 71 . "\n";
 
         print "COMMANDS TO INSTALL:\n\n";
-        foreach my $module (@missing_names) {
-            my $command = install_command($missing{$module});
-            printf "%15s: $command\n", $module;
+        foreach my $module (@missing) {
+            my $command = install_command($module);
+            printf "%15s: $command\n", $module->{package};
         }
     }
+
+    if ($output && $check_results->{any_missing} && !ON_WINDOWS) {
+        print install_string('install_all', { perl => $^X });
+    }
 }
 
 sub check_graphviz {
@@ -494,21 +511,6 @@ sub check_graphviz {
     return $return;
 }
 
-sub display_version_and_os {
-    # Display version information
-    printf "\n* This is Bugzilla " . BUGZILLA_VERSION . " on perl %vd\n",
-           $^V;
-    my @os_details = POSIX::uname;
-    # 0 is the name of the OS, 2 is the major version,
-    my $os_name = $os_details[0] . ' ' . $os_details[2];
-    if (ON_WINDOWS) {
-        require Win32;
-        $os_name = Win32::GetOSName();
-    }
-    # 3 is the minor version.
-    print "* Running on $os_name $os_details[3]\n"
-}
-
 # This was originally clipped from the libnet Makefile.PL, adapted here to
 # use the below vers_cmp routine for accurate version checking.
 sub have_vers {
@@ -521,15 +523,10 @@ sub have_vers {
     }
     my $wanted  = $params->{version};
 
-    my ($msg, $vnum, $vstr);
-    no strict 'refs';
-    printf("Checking for %15s %-9s ", $package, !$wanted?'(any)':"(v$wanted)") 
-        if $output;
-
     eval "require $module;";
 
     # VERSION is provided by UNIVERSAL::
-    $vnum = eval { $module->VERSION } || -1;
+    my $vnum = eval { $module->VERSION } || -1;
 
     # CGI's versioning scheme went 2.75, 2.751, 2.752, 2.753, 2.76
     # That breaks the standard version tests, so we need to manually correct
@@ -538,14 +535,15 @@ sub have_vers {
         $vnum = $1 . "." . $2;
     }
 
+    my $vstr;
     if ($vnum eq "-1") { # string compare just in case it's non-numeric
-        $vstr = "not found";
+        $vstr = install_string('module_not_found');
     }
     elsif (vers_cmp($vnum,"0") > -1) {
-        $vstr = "found v$vnum";
+        $vstr = install_string('module_found', { ver => $vnum });
     }
     else {
-        $vstr = "found unknown version";
+        $vstr = install_string('module_unknown_version');
     }
 
     my $vok = (vers_cmp($vnum,$wanted) > -1);
@@ -555,53 +553,17 @@ sub have_vers {
         $vok = 0 if $blacklisted;
     }
 
-    my $ok = $vok ? "ok:" : "";
-    my $black_string = $blacklisted ? "(blacklisted)" : "";
-    print "$ok $vstr $black_string\n" if $output;
-    return $vok ? 1 : 0;
-}
+    if ($output) {
+        my $ok           = $vok ? install_string('module_ok') : '';
+        my $black_string = $blacklisted ? install_string('blacklisted') : '';
+        my $want_string  = $wanted ? "v$wanted" : install_string('any');
 
-# This is taken straight from Sort::Versions 1.5, which is not included
-# with perl by default.
-sub vers_cmp {
-    my ($a, $b) = @_;
-
-    # Remove leading zeroes - Bug 344661
-    $a =~ s/^0*(\d.+)/$1/;
-    $b =~ s/^0*(\d.+)/$1/;
-
-    my @A = ($a =~ /([-.]|\d+|[^-.\d]+)/g);
-    my @B = ($b =~ /([-.]|\d+|[^-.\d]+)/g);
-
-    my ($A, $B);
-    while (@A and @B) {
-        $A = shift @A;
-        $B = shift @B;
-        if ($A eq '-' and $B eq '-') {
-            next;
-        } elsif ( $A eq '-' ) {
-            return -1;
-        } elsif ( $B eq '-') {
-            return 1;
-        } elsif ($A eq '.' and $B eq '.') {
-            next;
-        } elsif ( $A eq '.' ) {
-            return -1;
-        } elsif ( $B eq '.' ) {
-            return 1;
-        } elsif ($A =~ /^\d+$/ and $B =~ /^\d+$/) {
-            if ($A =~ /^0/ || $B =~ /^0/) {
-                return $A cmp $B if $A cmp $B;
-            } else {
-                return $A <=> $B if $A <=> $B;
-            }
-        } else {
-            $A = uc $A;
-            $B = uc $B;
-            return $A cmp $B if $A cmp $B;
-        }
+        $ok = "$ok:" if $ok;
+        printf "%s %19s %-9s $ok $vstr $black_string\n",
+            install_string('checking_for'), $package, "($want_string)";
     }
-    @A <=> @B;
+    
+    return $vok ? 1 : 0;
 }
 
 sub install_command {
@@ -613,7 +575,7 @@ sub install_command {
         $package = $module->{package};
     }
     else {
-        $command = "$^X -MCPAN -e 'install \"\%s\"'";
+        $command = "$^X install-module.pl \%s";
         # Non-Windows installations need to use module names, because
         # CPAN doesn't understand package names.
         $package = $module->{module};
@@ -621,7 +583,6 @@ sub install_command {
     return sprintf $command, $package;
 }
 
-
 1;
 
 __END__
@@ -653,26 +614,46 @@ represent the name of the module and the version that we require.
 
 =over 4
 
-=item C<check_requirements($output)>
+=item C<check_requirements>
 
- Description: This checks what optional or required perl modules
-              are installed, like C<checksetup.pl> does.
+=over
 
- Params:      C<$output> - C<true> if you want the function to print
-                           out information about what it's doing,
-                           and the versions of everything installed.
-                           If you don't pass the minimum requirements,
-                           the will always print out something, 
-                           regardless of this parameter.
+=item B<Description>
 
- Returns:    A hashref containing three values:
-             C<pass> - Whether or not we have all the mandatory 
-                       requirements.
-             C<missing> - A hash showing which mandatory requirements
-                          are missing. The key is the module name,
-                          and the value is the version we require.
-             C<optional> - Which optional modules are installed and
-                           up-to-date enough for Bugzilla.
+This checks what optional or required perl modules are installed, like
+C<checksetup.pl> does.
+
+=item B<Params>
+
+=over
+
+=item C<$output> - C<true> if you want the function to print out information
+about what it's doing, and the versions of everything installed.
+
+=back
+
+=item B<Returns>
+
+A hashref containing these values:
+
+=over
+
+=item C<pass> - Whether or not we have all the mandatory requirements.
+
+=item C<missing> - An arrayref containing any required modules that
+are not installed or that are not up-to-date. Each item in the array is
+a hashref in the format of items from L</REQUIRED_MODULES>.
+
+=item C<optional> - The same as C<missing>, but for optional modules.
+
+=item C<have_one_dbd> - True if at least one C<DBD::> module is installed.
+
+=item C<any_missing> - True if there are any missing modules, even optional
+modules.
+
+=back
+
+=back
 
 =item C<check_graphviz($output)>
 
@@ -684,18 +665,6 @@ Params:      C<$output> - C<$true> if you want the function to
 
 Returns:     C<1> if the check was successful, C<0> otherwise.
 
-=item C<vers_cmp($a, $b)>
-
- Description: This is a comparison function, like you would use in
-              C<sort>, except that it compares two version numbers.
-              It's actually identical to versioncmp from 
-              L<Sort::Versions>.
-
- Params:      c<$a> and C<$b> are versions you want to compare.
-
- Returns:     -1 if $a is less than $b, 0 if they are equal, and
-              1 if $a is greater than $b.
-
 =item C<have_vers($module, $output)>
 
  Description: Tells you whether or not you have the appropriate
